@@ -1720,14 +1720,15 @@ static int get_header(FILE *f, struct lha_data *data)
 		read8(f);		/* skip OS id */
 		while ((size = read16l(f)) != 0) {
 			int type = read8(f);
+			int s = size - 3;
 			if (type == 0x01) {
 				/* Sanity check */
-				if (size - 3 > 256)
+				if (s < 0 || s > 256)
 					return -1;
 
-				fread(data->name, 1, size - 3, f);
+				fread(data->name, 1, s, f);
 			} else {
-				fseek(f, size - 3, SEEK_CUR);
+				fseek(f, s, SEEK_CUR);
 			}
 		}
 		break;
