@@ -1,6 +1,8 @@
 #ifndef LIBXMP_LFO_H
 #define LIBXMP_LFO_H
 
+#include "common.h"
+
 struct lfo {
 	int type;
 	int rate;
@@ -9,7 +11,7 @@ struct lfo {
 };
 
 
-int get_lfo(struct lfo *, int);
+int get_lfo(struct context_data *, struct lfo *, int, int);
 void update_lfo(struct lfo *);
 void set_lfo_phase(struct lfo *, int);
 void set_lfo_depth(struct lfo *, int);
