@@ -162,6 +162,11 @@ static void check_envelope(struct xmp_envelope *env)
 	if (env->lps >= env->npt || env->lpe >= env->npt) {
 		env->flg &= ~XMP_ENVELOPE_LOOP;
 	}
+
+	/* Disable envelope loop if invalid sustain */
+	if (env->sus >= env->npt) {
+		env->flg &= ~XMP_ENVELOPE_ON;
+	}
 }
 
 void load_prologue(struct context_data *ctx)
@@ -174,7 +179,7 @@ void load_prologue(struct context_data *ctx)
 	m->rrate = PAL_RATE;
 	m->c4rate = C4_PAL_RATE;
 	m->volbase = 0x40;
-	m->gvolbase = 0x40;
+	m->gvol = m->gvolbase = 0x40;
 	m->vol_table = NULL;
 	m->quirk = 0;
 	m->read_event_type = READ_EVENT_MOD;
@@ -213,13 +218,13 @@ void load_epilogue(struct context_data *ctx)
 	struct xmp_module *mod = &m->mod;
 	int i, j;
 
-    	mod->gvl = m->gvolbase;
+    	mod->gvl = m->gvol;
 
 	/* Sanity check for module parameters */
-	CLAMP(mod->len, 0, 255);
-	CLAMP(mod->pat, 0, 255);
+	CLAMP(mod->len, 0, XMP_MAX_MOD_LENGTH);
+	CLAMP(mod->pat, 0, 256);
 	CLAMP(mod->ins, 0, 255);
-	CLAMP(mod->smp, 0, 255);
+	CLAMP(mod->smp, 0, MAX_SAMPLES);
 	CLAMP(mod->chn, 0, XMP_MAX_CHANNELS);
 
 	/* Fix cases where the restart value is invalid e.g. kc_fall8.xm
