@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -40,7 +40,13 @@ static int ssn_test(HIO_HANDLE *f, char *t, const int start)
     if (id != 0x6966 && id != 0x4a4e)
 	return -1;
 
-    hio_seek(f, 238, SEEK_CUR);
+    hio_seek(f, 110, SEEK_SET);
+    if (hio_read8(f) > 64)
+	return -1;
+    if (hio_read8(f) > 128)
+	return -1;
+
+    hio_seek(f, 240, SEEK_SET);
     if (hio_read8(f) != 0xff)
 	return -1;
 
@@ -99,18 +105,27 @@ static int ssn_load(struct module_data *m, HIO_HANDLE *f, const int start)
     hio_read(&sfh.message, 108, 1, f);	/* Song message */
     sfh.nos = hio_read8(f);		/* Number of samples (0-64) */
     sfh.nop = hio_read8(f);		/* Number of patterns (0-128) */
+
+    /* Sanity check */
+    if (sfh.nop > 128)
+	return -1;
+
     sfh.loop = hio_read8(f);		/* Loop order number */
-    hio_read(&sfh.order, 128, 1, f);	/* Order list */
-    hio_read(&sfh.speed, 128, 1, f);	/* Tempo list for patterns */
-    hio_read(&sfh.pbrk, 128, 1, f);	/* Break list for patterns */
+    if (hio_read(&sfh.order, 1, 128, f) != 128)	/* Order list */
+	return -1;
+    if (hio_read(&sfh.speed, 1, 128, f) != 128)	/* Tempo list for patterns */
+	return -1;
+    if (hio_read(&sfh.pbrk, 1, 128, f) != 128) 	/* Break list for patterns */
+	return -1;
 
     mod->chn = 8;
     mod->ins = sfh.nos;
     mod->pat = sfh.nop;
     mod->trk = mod->chn * mod->pat;
-    for (i = 0; i < 128; i++)
+    for (i = 0; i < 128; i++) {
 	if (sfh.order[i] > sfh.nop)
 	    break;
+    }
     mod->len = i;
     memcpy (mod->xxo, sfh.order, mod->len);
     mod->spd = 6;
@@ -137,30 +152,41 @@ static int ssn_load(struct module_data *m, HIO_HANDLE *f, const int start)
     D_(D_INFO "Instruments: %d", mod->pat);
 
     for (i = 0; i < mod->ins; i++) {
+	struct xmp_instrument *xxi = &mod->xxi[i];
+	struct xmp_sample *xxs = &mod->xxs[i];
+	struct xmp_subinstrument *sub;
+
 	if (subinstrument_alloc(mod, i, 1) < 0)
 	    return -1;
 
+	sub = &xxi->sub[0];
+
 	hio_read (&sih.name, 13, 1, f);		/* ASCIIZ instrument name */
 	sih.length = hio_read32l(f);		/* Instrument size */
 	sih.loop_start = hio_read32l(f);	/* Instrument loop start */
 	sih.loopend = hio_read32l(f);		/* Instrument loop end */
 
-	mod->xxs[i].len = sih.length;
-	mod->xxs[i].lps = sih.loop_start;
-	mod->xxs[i].lpe = sih.loopend >= 0xfffff ? 0 : sih.loopend;
-	mod->xxs[i].flg = mod->xxs[i].lpe ? XMP_SAMPLE_LOOP : 0;	/* 1 == Forward loop */
-	mod->xxi[i].sub[0].vol = 0x40;
-	mod->xxi[i].sub[0].pan = 0x80;
-	mod->xxi[i].sub[0].sid = i;
+	/* Sanity check */
+	if (sih.length > MAX_SAMPLE_SIZE)
+	    return -1;
 
-	if (mod->xxs[i].len > 0)
-		mod->xxi[i].nsm = 1;
+	xxs->len = sih.length;
+	xxs->lps = sih.loop_start;
+	xxs->lpe = sih.loopend >= 0xfffff ? 0 : sih.loopend;
+	xxs->flg = xxs->lpe ? XMP_SAMPLE_LOOP : 0;	/* 1 == Forward loop */
+
+	sub->vol = 0x40;
+	sub->pan = 0x80;
+	sub->sid = i;
+
+	if (xxs->len > 0)
+		xxi->nsm = 1;
 
 	instrument_name(mod, i, sih.name, 13);
 
 	D_(D_INFO "[%2X] %-14.14s %04x %04x %04x %c", i,
-		mod->xxi[i].name, mod->xxs[i].len, mod->xxs[i].lps, mod->xxs[i].lpe,
-		mod->xxs[i].flg & XMP_SAMPLE_LOOP ? 'L' : ' ');
+		xxi->name, xxs->len, xxs->lps, xxs->lpe,
+		xxs->flg & XMP_SAMPLE_LOOP ? 'L' : ' ');
     }
 
     if (pattern_init(mod) < 0)
@@ -169,13 +195,22 @@ static int ssn_load(struct module_data *m, HIO_HANDLE *f, const int start)
     /* Read and convert patterns */
     D_(D_INFO "Stored patterns: %d", mod->pat);
     for (i = 0; i < mod->pat; i++) {
+	int pbrk;
+
 	if (pattern_tracks_alloc(mod, i, 64) < 0)
 	    return -1;
 
-	EVENT(i, 0, 0).f2t = FX_SPEED_CP;
-	EVENT(i, 0, 0).f2p = sfh.speed[i];
-	EVENT(i, 1, sfh.pbrk[i]).f2t = FX_BREAK;
-	EVENT(i, 1, sfh.pbrk[i]).f2p = 0;
+	event = &EVENT(i, 0, 0);
+	event->f2t = FX_SPEED_CP;
+	event->f2p = sfh.speed[i];
+
+	pbrk = sfh.pbrk[i];
+	if (pbrk >= 64)
+	    return -1;
+
+	event = &EVENT(i, 1, pbrk);
+	event->f2t = FX_BREAK;
+	event->f2p = 0;
 
 	for (j = 0; j < 64 * 8; j++) {
 	    event = &EVENT(i, j % 8, j / 8);
