@@ -376,6 +376,9 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
     mod->smp = ifh.smpnum;
     mod->pat = ifh.patnum;
 
+    memset(lastevent, 0, L_CHANNELS * sizeof (struct xmp_event));
+    memset(&dummy, 0, sizeof (struct xmp_event));
+
     /* Sanity check */
     if (mod->ins > 255 || mod->smp > 255 || mod->pat > 255) {
 	goto err;
@@ -579,7 +582,10 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	    i2h.nos = hio_read8(f);
 	    i2h.rsvd1 = hio_read8(f);
-	    hio_read(&i2h.name, 26, 1, f);
+
+            if (hio_read(&i2h.name, 1, 26, f) != 26) {
+                goto err4;
+            }
 
 	    fix_name(i2h.name, 26);
 
@@ -588,7 +594,10 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	    i2h.mch = hio_read8(f);
 	    i2h.mpr = hio_read8(f);
 	    i2h.mbnk = hio_read16l(f);
-	    hio_read(&i2h.keys, 240, 1, f);
+
+            if (hio_read(&i2h.keys, 1, 240, f) != 240) {
+                goto err4; 
+            }
 
 	    copy_adjust(xxi->name, i2h.name, 25);
 	    xxi->rls = i2h.fadeout << 6;
@@ -629,7 +638,7 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	    for (k = j = 0; j < 120; j++) {
 		c = i2h.keys[j * 2 + 1] - 1;
-		if (c < 0) {
+		if (c < 0 || c >= 120) {
 		    xxi->map[j].ins = 0xff;	/* No sample */
 		    xxi->map[j].xpo = 0;
 		    continue;
@@ -703,8 +712,9 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	    i1h.nos = hio_read8(f);
 	    i1h.rsvd2 = hio_read8(f);
 
-	    if (hio_read(&i1h.name, 1, 26, f) != 26)
-		goto err4;
+	    if (hio_read(&i1h.name, 1, 26, f) != 26) {
+                goto err4;
+            }
 
 	    fix_name(i1h.name, 26);
 
@@ -823,7 +833,10 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	ish.gvl = hio_read8(f);
 	ish.flags = hio_read8(f);
 	ish.vol = hio_read8(f);
-	hio_read(&ish.name, 26, 1, f);
+
+	if (hio_read(&ish.name, 1, 26, f) != 26) {
+	    goto err4;
+	}
 
 	fix_name(ish.name, 26);
 
@@ -976,7 +989,7 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	hio_seek(f, start + pp_pat[i], SEEK_SET);
 	pat_len = hio_read16l(f) /* - 4*/;
 	hio_read16l(f);
-	memset (mask, 0, L_CHANNELS);
+	memset(mask, 0, L_CHANNELS);
 	hio_read16l(f);
 	hio_read16l(f);
 
@@ -1052,7 +1065,7 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	if (tracks_in_pattern_alloc(mod, i) < 0)
 	    goto err4;
 
-	memset (mask, 0, L_CHANNELS);
+	memset(mask, 0, L_CHANNELS);
 	hio_read16l(f);
 	hio_read16l(f);
 
@@ -1074,7 +1087,12 @@ static int it_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	     * real number of channels before loading the patterns and
 	     * we don't want to set it to 64 channels.
 	     */
-	    event = c >= mod->chn || r >= mod->xxp[i]->rows ? &dummy : &EVENT (i, c, r);
+            if (c >= mod->chn || r >= mod->xxp[i]->rows) {
+                event = &dummy;
+            } else {
+                event = &EVENT(i, c, r);
+            }
+
 	    if (mask[c] & 0x01) {
 		b = hio_read8(f);
 
