@@ -334,6 +334,11 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		hio_seek(f, size + plen - 4, SEEK_CUR);
 	}
 
+	/* Sanity check */
+	if (mod->chn > 16) {
+		return -1;
+	}
+
 	mod->trk = mod->chn * mod->pat;
 
 	if (pattern_init(mod) < 0)
@@ -419,6 +424,9 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		for (j = 0; j < 32; j++) {
 			int line = y * 32 + j;
 
+			if (line >= rows)
+				break;
+
 			if (linemask[y] & 0x80000000) {
 				chmsk = stream_read_aligned16(&stream, chn);
 				for (k = 0; k < chn; k++, chmsk <<= 1) {
@@ -566,6 +574,14 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			synth.volspeed = hio_read8(f);
 			synth.wfspeed = hio_read8(f);
 			synth.wforms = hio_read16b(f);
+
+			/* Sanity check */
+			if (synth.voltbllen > 128 ||
+			    synth.wftbllen > 128 ||
+			    synth.wforms > 256) {
+				return -1;
+			}
+
 			hio_read(synth.voltbl, 1, synth.voltbllen, f);;
 			hio_read(synth.wftbl, 1, synth.wftbllen, f);;
 
@@ -627,6 +643,14 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			synth.volspeed = hio_read8(f);
 			synth.wfspeed = hio_read8(f);
 			synth.wforms = hio_read16b(f);
+
+			/* Sanity check */
+			if (synth.voltbllen > 128 ||
+			    synth.wftbllen > 128 ||
+			    synth.wforms > 256) {
+				return -1;
+			}
+
 			hio_read(synth.voltbl, 1, synth.voltbllen, f);;
 			hio_read(synth.wftbl, 1, synth.wftbllen, f);;
 			if (synth.wforms == 0xffff)	
@@ -702,6 +726,10 @@ static int med4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		sub->xpo = temp_inst[i].transpose;
 		sub->sid = smp_idx;
 
+		/* Sanity check */
+		if (smp_idx >= mod->smp)
+			return -1;
+
 		xxs = &mod->xxs[smp_idx];
 
 		xxs->len = length;
