@@ -256,6 +256,7 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
     int tracker_id = TRACKER_PROTRACKER;
     int has_loop_0 = 0;
     int has_vol_in_empty_ins = 0;
+    int out_of_range = 0;
 
     LOAD_INIT();
 
@@ -293,7 +294,7 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	}
     }
 
-    if (!mod->chn) {
+    if (mod->chn == 0) {
 	if (!strncmp(magic + 2, "CH", 2) &&
 	    isdigit((int)magic[0]) && isdigit((int)magic[1])) {
 	    mod->chn = (*magic - '0') * 10 + magic[1] - '0';
@@ -527,7 +528,7 @@ skip_test:
     mod->trk = mod->chn * mod->pat;
 
     for (i = 0; i < mod->ins; i++) {
-	D_(D_INFO "[%2X] %-22.22s %04x %04x %04x %c V%02x %+d %c\n",
+	D_(D_INFO "[%2X] %-22.22s %04x %04x %04x %c V%02x %+d %c",
 		i, mod->xxi[i].name,
 		mod->xxs[i].len, mod->xxs[i].lps, mod->xxs[i].lpe,
 		(mh.ins[i].loop_size > 1 && mod->xxs[i].lpe > 8) ?
@@ -559,12 +560,14 @@ skip_test:
 
 	    /* Check out-of-range notes in Amiga trackers */
 	    period = ((int)(LSN(mod_event[0])) << 8) | mod_event[1];
-	    if (period != 0 && (period < 108 || period > 907) && (
-			tracker_id == TRACKER_PROTRACKER ||
-			tracker_id == TRACKER_NOISETRACKER ||
-			tracker_id == TRACKER_PROBABLY_NOISETRACKER ||
-			tracker_id == TRACKER_SOUNDTRACKER)) {	/* note > B-3 */
-		tracker_id = TRACKER_UNKNOWN;
+	    if (period != 0 && (period < 108 || period > 907)) {
+		out_of_range = 1;
+		if (tracker_id == TRACKER_PROTRACKER ||
+		    tracker_id == TRACKER_NOISETRACKER ||
+		    tracker_id == TRACKER_PROBABLY_NOISETRACKER ||
+		    tracker_id == TRACKER_SOUNDTRACKER) {   /* note > B-3 */
+			tracker_id = TRACKER_UNKNOWN;
+		}
 	    }
 
 	    /* Filter noisetracker events */
@@ -655,6 +658,10 @@ skip_test:
 	break;
     }
 
+    if (out_of_range) {
+	m->quirk &= ~QUIRK_MODRNG;
+    }
+
     if (tracker_id == TRACKER_MODSGRAVE) {
 	snprintf(mod->type, XMP_NAME_SIZE, "%s", tracker);
     } else {
