@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -39,10 +39,46 @@ const struct format_loader no_loader = {
 
 static int no_test(HIO_HANDLE *f, char *t, const int start)
 {
+	int nsize, pat, chn;
+	int i;
+
+	hio_seek(f, start, SEEK_CUR);
+
 	if (hio_read32b(f) != 0x4e4f0000)		/* NO 0x00 0x00 */
 		return -1;
 
-	read_title(f, t, hio_read8(f));
+	nsize = hio_read8(f);
+	if (nsize != 20)
+		return -1;
+
+	/* test title */
+	for (i = 0; i < nsize; i++) {
+		if (hio_read8(f) == 0)
+			return -1;
+	}
+
+	hio_seek(f, 9, SEEK_CUR);
+
+	/* test number of patterns */
+	pat = hio_read8(f);
+	if (pat == 0)
+		return -1;
+
+	hio_read8(f);
+
+	/* test number of channels */
+	chn = hio_read8(f);
+	if (chn <= 0 || chn > 16)
+		return -1;
+
+	hio_seek(f, start + 5, SEEK_SET);
+
+	if (nsize > XMP_NAME_SIZE) {
+		read_title(f, t, XMP_NAME_SIZE);
+		hio_seek(f, nsize - XMP_NAME_SIZE, SEEK_CUR);
+	} else {
+		read_title(f, t, nsize);
+	}
 
 	return 0;
 }
