@@ -3,7 +3,7 @@
  *
  * Converts NoisePacked MODs back to ptk
  *
- * Modified in 2006,2007,2014 by Claudio Matsuoka
+ * Modified in 2006,2007,2014,2015 by Claudio Matsuoka
  */
 
 #include <string.h>
@@ -27,25 +27,25 @@ static int depack_np1(HIO_HANDLE *in, FILE *out)
 	memset(ptable, 0, 128);
 	memset(trk_addr, 0, 128 * 4 * 4);
 
-	c1 = hio_read8(in);				/* read number of samples */
+	c1 = hio_read8(in);			/* read number of samples */
 	c2 = hio_read8(in);
 	nins = ((c1 << 4) & 0xf0) | ((c2 >> 4) & 0x0f);
 
 	pw_write_zero(out, 20);			/* write title */
 
-	len = hio_read16b(in) / 2;			/* size of pattern list */
-	hio_read16b(in);				/* 2 unknown bytes */
+	len = hio_read16b(in) >> 1;		/* size of pattern list */
+	hio_read16b(in);			/* 2 unknown bytes */
 	/*tsize =*/ hio_read16b(in);		/* read track data size */
 
 	/* read sample descriptions */
 	for (i = 0; i < nins; i++) {
-		hio_read32b(in);			/* bypass 4 unknown bytes */
+		hio_read32b(in);		/* bypass 4 unknown bytes */
 		pw_write_zero(out, 22);		/* sample name */
 		write16b(out, size = hio_read16b(in));	/* size */
 		ssize += size * 2;
-		write8(out, hio_read8(in));		/* finetune */
-		write8(out, hio_read8(in));		/* volume */
-		hio_read32b(in);			/* bypass 4 unknown bytes */
+		write8(out, hio_read8(in));	/* finetune */
+		write8(out, hio_read8(in));	/* volume */
+		hio_read32b(in);		/* bypass 4 unknown bytes */
 		size = hio_read16b(in);		/* read loop size */
 		write16b(out, hio_read16b(in) / 2);	/* loop start */
 		write16b(out, size);		/* write loop size */
@@ -54,14 +54,15 @@ static int depack_np1(HIO_HANDLE *in, FILE *out)
 	/* fill up to 31 samples */
 	memset(tmp, 0, 30);
 	tmp[29] = 0x01;
-	for (; i < 31; i++)
+	for (; i < 31; i++) {
 		fwrite(tmp, 30, 1, out);
+	}
 
 	write8(out, len);		/* write size of pattern list */
 	write8(out, 0x7f);		/* write noisetracker byte */
 
-	hio_seek(in, 2, SEEK_CUR);		/* always $02? */
-	hio_seek(in, 2, SEEK_CUR);		/* unknown */
+	hio_seek(in, 2, SEEK_CUR);	/* always $02? */
+	hio_seek(in, 2, SEEK_CUR);	/* unknown */
 
 	/* read pattern table */
 	npat = 0;
@@ -141,15 +142,18 @@ static int test_np1(uint8 *data, char *t, int s)
 	int num_ins, ssize, hdr_size, ptab_size, trk_size, max_pptr;
 	int i;
 
+	PW_REQUEST_DATA(s, 10);
+
 	/* size of the pattern table */
 	ptab_size = readmem16b(data + 2);
-	if (ptab_size == 0 || ptab_size & 1)
+	if (ptab_size == 0 || ptab_size & 1 || ptab_size > 0xff)
 		return -1;
 
-	/* test nbr of samples */
+	/* test number of samples */
 	if ((data[1] & 0x0f) != 0x0c)
 		return -1;
 
+	/* number of samples */
 	num_ins = ((data[0] << 4) & 0xf0) | ((data[1] >> 4) & 0x0f);
 	if (num_ins == 0 || num_ins > 0x1f)
 		return -1;
@@ -189,11 +193,13 @@ static int test_np1(uint8 *data, char *t, int s)
 	/* size of the header til the end of sample descriptions */
 	hdr_size = num_ins * 16 + 8 + 4;
 
+	PW_REQUEST_DATA(s, hdr_size + ptab_size + 2);
+
 	/* test pattern table */
 	max_pptr = 0;
 	for (i = 0; i < ptab_size; i += 2) {
 		int pptr = readmem16b(data + hdr_size + i);
-		if (pptr % 8)
+		if (pptr & 0x07 || pptr >= 0x400)
 			return -1;
 		if (pptr > max_pptr)
 			max_pptr = pptr;
