@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -56,7 +56,7 @@ int pw_test_format(HIO_HANDLE *f, char *t, const int start,
 	if (b == NULL)
 		return -1;
 
-	hio_read(b, s, 1, f);
+	s = hio_read(b, 1, s, f);
 
 	while ((extra = pw_check(b, s, info)) > 0) {
 		unsigned char *buf = realloc(b, s + extra);
@@ -65,7 +65,12 @@ int pw_test_format(HIO_HANDLE *f, char *t, const int start,
 			return -1;
 		}
 		b = buf;
-		hio_read(b + s, extra, 1, f);
+
+		if (hio_read(b + s, extra, 1, f) == 0) {
+			free(b);
+			return -1;
+		}
+
 		s += extra;
 	}
 
@@ -96,13 +101,17 @@ static int pw_load(struct module_data *m, HIO_HANDLE *h, const int start)
 	if ((temp = make_temp_file(&temp_name)) == NULL)
 		goto err;
 
-	if (pw_wizardry(h, temp, &name) < 0)
+	if (pw_wizardry(h, temp, &name) < 0) {
+		fclose(temp);
 		goto err2;
+	}
 	
 	/* Module loading */
 
-	if ((f = hio_open_file(temp)) == NULL)
+	if ((f = hio_open_file(temp)) == NULL) {
+		fclose(temp);
 		goto err2;
+	}
 
 	LOAD_INIT();
 
@@ -131,8 +140,6 @@ static int pw_load(struct module_data *m, HIO_HANDLE *h, const int start)
 	memcpy(mod->xxo, mh.order, 128);
 
 	for (i = 0; i < 128; i++) {
-		if (mod->chn > 4)
-			mod->xxo[i] >>= 1;
 		if (mod->xxo[i] > mod->pat)
 			mod->pat = mod->xxo[i];
 	}
@@ -209,7 +216,6 @@ static int pw_load(struct module_data *m, HIO_HANDLE *h, const int start)
     err3:
 	hio_close(f);
     err2:
-	fclose(temp);
 	unlink_temp_file(temp_name);
     err:
 	return -1;
