@@ -98,8 +98,8 @@ static int s3m_test(HIO_HANDLE *f, char *t, const int start)
     if (hio_read32b(f) != MAGIC_SCRM)
 	return -1;
 
-    hio_seek(f, start + 28, SEEK_SET);
-    if (hio_read16b(f) != 0x1a10)
+    hio_seek(f, start + 29, SEEK_SET);
+    if (hio_read8(f) != 0x10)
 	return -1;
 
     hio_seek(f, start + 0, SEEK_SET);
