@@ -264,6 +264,11 @@ static int sym_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	mod->chn = hio_read8(f);
 	mod->len = mod->pat = hio_read16l(f);
+
+	/* Sanity check */
+	if (mod->chn > 8 || mod->pat > 256)
+		return -1;
+
 	mod->trk = hio_read16l(f);	/* Symphony patterns are actually tracks */
 	infolen = hio_read24l(f);
 
@@ -281,12 +286,21 @@ static int sym_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		if (~sn[i] & 0x80) {
 			mod->xxs[i].len = hio_read24l(f) << 1;
 			mod->xxi[i].nsm = 1;
+
+			/* Sanity check */
+			if (mod->xxs[i].len > 0x80000)
+				return -1;
 		}
 	}
 
 	a = hio_read8(f);		/* track name length */
+	if (a > 32) {
+		hio_read(mod->name, 1, 32, f);
+		hio_seek(f, a - 32, SEEK_SET);
+	} else {
+		hio_read(mod->name, 1, a, f);
+	}
 
-	hio_read(mod->name, 1, a, f);
 	hio_read(&allowed_effects, 1, 8, f);
 
 	MODULE_INFO();
@@ -315,7 +329,10 @@ static int sym_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			return -1;
 		}
 	} else {
-		hio_read(buf, 1, size, f);
+		if (hio_read(buf, 1, size, f) != size) {
+			free(buf);
+			return -1;
+		}
 	}
 
 	for (i = 0; i < mod->len; i++) {	/* len == pat */
@@ -327,11 +344,18 @@ static int sym_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		for (j = 0; j < mod->chn; j++) {
 			int idx = 2 * (i * mod->chn + j);
-			mod->xxp[i]->index[j] = readptr16l(&buf[idx]);
+			int t = readptr16l(&buf[idx]);
 
-			if (mod->xxp[i]->index[j] == 0x1000) /* empty trk */
-				mod->xxp[i]->index[j] = mod->trk - 1;
+			/* Sanity check */
+			if (t >= mod->trk - 1) {
+				free(buf);
+				return -1;
+			}
+	
+			if (t == 0x1000) /* empty trk */
+				t = mod->trk - 1;
 
+			mod->xxp[i]->index[j] = t;
 		}
 		mod->xxo[i] = i;
 	}
@@ -359,7 +383,10 @@ static int sym_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			return -1;
 		}
 	} else {
-		hio_read(buf, 1, size, f);
+		if (hio_read(buf, 1, size, f) != size) {
+			free(buf);
+			return -1;
+		}
 	}
 
 	for (i = 0; i < mod->trk - 1; i++) {
