@@ -340,7 +340,7 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 	/* Sanity check */
 	if (xih.samples > 0x10 || (xih.samples > 0 && xih.sh_size > 0x100)) {
 		D_(D_CRIT "Sanity check: %d %d", xih.samples, xih.sh_size);
-		break;
+		return -1;
 	}
 
 	instrument_name(mod, i, xih.name, 22);
@@ -568,11 +568,6 @@ static int xm_load(struct module_data *m, HIO_HANDLE *f, const int start)
         return -1;
     }
 
-    if (xfh.tempo < 0) {
-	D_(D_CRIT "Sanity check: %d", xfh.tempo);
-        return -1;
-    }
-
     if (xfh.tempo >= 32 || xfh.bpm < 32 || xfh.bpm > 255) {
         if (memcmp("MED2XM", xfh.tracker, 6)) {
 	    D_(D_CRIT "Sanity check: %d %d", xfh.tempo, xfh.bpm);
