@@ -130,8 +130,9 @@ static void reset_channels(struct context_data *ctx)
 	struct player_data *p = &ctx->p;
 	struct module_data *m = &ctx->m;
 	struct xmp_module *mod = &m->mod;
+	struct smix_data *smix = &ctx->smix;
 	struct channel_data *xc;
-	int i;
+	int i, j;
 
 #ifndef LIBXMP_CORE_PLAYER
 	m->synth->reset(ctx);
@@ -162,13 +163,28 @@ static void reset_channels(struct context_data *ctx)
 
 	for (i = 0; i < p->virt.num_tracks; i++) {
 		xc = &p->xc_data[i];
-		xc->pan.val = mod->xxc[i].pan;
-		xc->mastervol = mod->xxc[i].vol;
 		xc->filter.cutoff = 0xff;
+
+		if (i >= mod->chn && i < mod->chn + smix->chn) {
+			xc->mastervol = 0x40;
+			xc->pan.val = 0x80;
+		} else {
+			xc->mastervol = mod->xxc[i].vol;
+			xc->pan.val = mod->xxc[i].pan;
+		}
 		
 		/* Amiga split channel */
 		if (mod->xxc[i].flg & XMP_CHANNEL_SPLIT) {
 			xc->split = ((mod->xxc[i].flg & 0x30) >> 4) + 1;
+			/* Connect split channel pairs */
+			for (j = 0; j < i; j++) {
+				if (mod->xxc[j].flg & XMP_CHANNEL_SPLIT) {
+					if (p->xc_data[j].split == xc->split) {
+						p->xc_data[j].pair = i;
+						xc->pair = j;
+					}
+				}
+			}
 		} else {
 			xc->split = 0;
 		}
@@ -437,22 +453,13 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 	else
 		finalvol = finalvol * p->smix_vol / 100;
 
-	xc->info_finalvol = TEST_NOTE(NOTE_SAMPLE_END) ? 0 :finalvol;
+	xc->info_finalvol = TEST_NOTE(NOTE_SAMPLE_END) ? 0 : finalvol;
+
+	virt_setvol(ctx, chn, finalvol);
 
 	/* Check Amiga split channel */
 	if (xc->split) {
-		int i;
-
-		/* Also set the volume of the paired channel */
-		for (i = 0; i < m->mod.chn; i++) {
-			if (i < chn && p->xc_data[i].split == xc->split) {
-				virt_setvol(ctx, i, finalvol);
-				virt_setvol(ctx, chn, finalvol);
-				break;
-			}
-		}
-	} else {
-		virt_setvol(ctx, chn, finalvol);
+		virt_setvol(ctx, xc->pair, finalvol);
 	}
 }
 
@@ -704,6 +711,10 @@ static void update_volume(struct context_data *ctx, int chn)
 	CLAMP(xc->volume, 0, m->volbase);
 	CLAMP(p->gvol, 0, m->gvolbase);
 	CLAMP(xc->mastervol, 0, m->volbase);
+
+	if (xc->split) {
+		p->xc_data[xc->pair].volume = xc->volume;
+	}
 }
 
 static void update_frequency(struct context_data *ctx, int chn)
@@ -874,7 +885,7 @@ static void play_channel(struct context_data *ctx, int chn)
 	update_pan(ctx, chn);
 
 #ifndef LIBXMP_CORE_PLAYER
-	if (HAS_QUIRK(QUIRK_PROTRACK)) {
+	if (HAS_QUIRK(QUIRK_PROTRACK) && xc->ins < mod->ins) {
 		update_invloop(m, xc);
 	}
 #endif
@@ -1248,7 +1259,7 @@ int xmp_play_frame(xmp_context opaque)
 		RESET(KEY_OFF);
 	}
 
-	/* check new r w */
+	/* check new row */
 
 	if (p->frame == 0) {			/* first frame in row */
 		check_end_of_module(ctx);
