@@ -120,8 +120,18 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 				struct xmp_subinstrument *sub,
 				struct channel_data *xc, int is_toneporta)
 {
+	struct module_data *m = &ctx->m;
+	struct xmp_module *mod = &m->mod;
+	struct smix_data *smix = &ctx->smix;
+	
 	if (sub != NULL && note >= 0) {
-		struct xmp_instrument *xxi = &ctx->m.mod.xxi[xc->ins];
+		struct xmp_instrument *xxi;
+
+		if (xc->ins >= mod->ins) {
+			xxi = &smix->xxi[xc->ins - mod->ins];
+		} else {
+			xxi = &mod->xxi[xc->ins];
+		}
 
 		xc->finetune = sub->fin;
 		xc->gvl = sub->gvl;
@@ -243,7 +253,12 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 				/* Get new instrument volume */
 				sub = get_subinstrument(ctx, ins, e->note - 1);
 				if (sub != NULL) {
-					xc->volume = sub->vol;
+					/* Dennis Lindroos: instrument volume
+					 * is not used on split channels
+					 */
+					if (!xc->split) {
+						xc->volume = sub->vol;
+					}
 					use_ins_vol = 0;
 				}
 			} else {
@@ -326,7 +341,7 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 		RESET(OFFSET);
 	}
 
-	if (use_ins_vol && !TEST(NEW_VOL)) {
+	if (use_ins_vol && !TEST(NEW_VOL) && !xc->split) {
 		xc->volume = sub->vol;
 	}
 
@@ -842,6 +857,32 @@ static int check_invalid_sample(struct context_data *ctx, int ins, int key)
 	return 0;
 }
 
+static void fix_period(struct context_data *ctx, int chn, struct xmp_subinstrument *sub)
+{
+	if (sub->nna == XMP_INST_NNA_CONT) {
+		struct player_data *p = &ctx->p;
+		struct module_data *m = &ctx->m;
+		struct channel_data *xc = &p->xc_data[chn];
+		struct xmp_instrument *xxi = get_instrument(ctx, xc->ins);
+
+		xc->period = note_to_period(xc->key + sub->xpo +
+			xxi->map[xc->key_porta].xpo, xc->finetune,
+			HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+	}
+}
+
+static int is_same_sid(struct context_data *ctx, int chn, int ins, int key)
+{
+	struct player_data *p = &ctx->p;
+	struct channel_data *xc = &p->xc_data[chn];
+	struct xmp_subinstrument *s1, *s2;
+
+	s1 = get_subinstrument(ctx, ins, key);
+	s2 = get_subinstrument(ctx, xc->ins, xc->key);
+
+	return (s1 && s2 && s1->sid == s2->sid);
+}
+
 static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 {
 	struct player_data *p = &ctx->p;
@@ -850,13 +891,14 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	struct channel_data *xc = &p->xc_data[chn];
 	int note, key;
 	struct xmp_subinstrument *sub;
-	int not_same_ins;
+	int not_same_ins, not_same_smp;
 	int new_invalid_ins;
 	int is_toneporta, is_release;
 	int candidate_ins;
 	int reset_env;
 	int use_ins_vol;
 	int sample_mode;
+	int toneporta_offset;
 	struct xmp_event ev;
 
 	memcpy(&ev, e, sizeof (struct xmp_event));
@@ -881,6 +923,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	note = -1;
 	key = ev.note;
 	not_same_ins = 0;
+	not_same_smp = 0;
 	new_invalid_ins = 0;
 	is_toneporta = 0;
 	is_release = 0;
@@ -888,6 +931,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	use_ins_vol = 0;
 	candidate_ins = xc->ins;
 	sample_mode = !HAS_QUIRK(QUIRK_VIRTUAL);
+	toneporta_offset = 0;
 
 	/* Notes with unmapped instruments are ignored */
 	if (ev.ins) {
@@ -921,11 +965,20 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		is_toneporta = 0;
 	}
 
+	/* Off-Porta.it */
+	if (is_toneporta && ev.fxt == FX_OFFSET) {
+		is_toneporta = 0;
+ 		if (!HAS_QUIRK(QUIRK_PRENV)) {
+			toneporta_offset = 1;
+			RESET_NOTE(NOTE_ENV_END);
+		}
+	}
+
 	/* Check instrument */
 
 	if (ev.ins) {
 		int ins = ev.ins - 1;
-		int set_new_ins = 0;
+		int set_new_ins = 1;
 
 		/* portamento_after_keyoff.it test case */
 		if (is_release && is_toneporta && !key) {
@@ -935,28 +988,17 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			}
 		}
 
-		if (!is_release) {
-			set_new_ins = 1;
-		}
-
-		if (is_toneporta) {
-			if (xc->ins != ins) {
-				set_new_ins = 1;
-			} else if (!HAS_QUIRK(QUIRK_PRENV)) {
-				/* also set if mapped sample is different,
-				 * see OpenMPT PortaInsNum.it
-				 */
-				struct xmp_subinstrument *s1, *s2;
-				s1 = get_subinstrument(ctx, ins, key - 1);
-				s2 = get_subinstrument(ctx, ins, xc->note);
-				if (s1 && s2 && s1->sid != s2->sid) {
-					set_new_ins = 1;
-					/* not same sample, actually */
-					not_same_ins = 1;
+		if (is_toneporta && xc->ins == ins) {
+			if (!HAS_QUIRK(QUIRK_PRENV)) {
+				if (is_same_sid(ctx, chn, ins, key - 1)) {
+					/* same instrument and same sample */
+					set_new_ins = !is_release;
+				} else {
+					/* same instrument, different sample */
+					not_same_ins = 1; /* need this too */
+					not_same_smp = 1;
 				}
 			}
-		} else {
-			set_new_ins = 1;
 		}
 
 		if (set_new_ins) {
@@ -987,13 +1029,9 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			}
 
 			if (xc->ins != ins && (!is_toneporta || !HAS_QUIRK(QUIRK_PRENV))) {
-				struct xmp_subinstrument *s1, *s2;
-				s1 = get_subinstrument(ctx, xc->ins, xc->key);
-				s2 = get_subinstrument(ctx, ins, key - 1);
-
 				candidate_ins = ins;
 
-				if (!s1 || !s2 || s1->sid != s2->sid) {
+				if (!is_same_sid(ctx, chn, ins, key - 1)) {
 					not_same_ins = 1;
 					if (is_toneporta) {
 						/* Get new instrument volume */
@@ -1082,12 +1120,15 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 				smp = -1;
 			}
 
-			to = virt_setpatch(ctx, chn, candidate_ins,
-				smp, note, sub->nna, sub->dct, sub->dca);
+			if (not_same_smp) {
+				fix_period(ctx, chn, sub);
+				virt_resetchannel(ctx, chn);
+			}
+			to = virt_setpatch(ctx, chn, candidate_ins, smp,
+				note, sub->nna, sub->dct, sub->dca);
 
 			if (to < 0)
 				return -1;
-
 			if (to != chn) {
 				copy_channel(p, to, chn);
 				p->xc_data[to].flags = 0;
@@ -1131,6 +1172,15 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		}
 	}
 
+	if (reset_env) {
+		if (ev.note) {
+			RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
+		}
+		/* Set after copying to new virtual channel (see ambio.it) */
+		xc->fadeout = 0x10000;
+	}
+
+
 	sub = get_subinstrument(ctx, xc->ins, xc->key);
 
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
@@ -1146,7 +1196,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 
 			if (TEST_NOTE(NOTE_CUT)) {
 				reset_envelopes(ctx, xc);
-			} else {
+			} else if (!toneporta_offset) {
 				reset_envelopes_carry(ctx, xc);
 			}
 			RESET_NOTE(NOTE_CUT);
@@ -1186,14 +1236,6 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		virt_voicepos(ctx, chn, xc->offset.val);
 	}
 
-	if (reset_env) {
-		if (ev.note) {
-			RESET_NOTE(NOTE_RELEASE|NOTE_SUSEXIT|NOTE_FADEOUT);
-		}
-		/* Set after copying to new virtual channel (see ambio.it) */
-		xc->fadeout = 0x10000;
-	}
-
 	if (use_ins_vol && !TEST(NEW_VOL)) {
 		int vol_swing = sub->rvv & 0xff;
 		CLAMP(vol_swing, 0, 100);
@@ -1390,10 +1432,11 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	RESET_NOTE(NOTE_RELEASE);
 
 	xc->ins = ins;
-	xc->ins_fade = mod->xxi[ins].rls;
 
-	if (ins >= mod->ins && ins < mod->ins + smix->ins)
+	if (ins >= mod->ins && ins < mod->ins + smix->ins) {
 		is_smix_ins = 1;
+		xc->ins_fade = smix->xxi[xc->ins - mod->ins].rls;
+	}
 
 	SET(NEW_NOTE);
 
